-- author: David Bartunek
-- Modul : Vsrcholovej modul na Diplomovou praci 
-- Vstup    :    top_clk                 -> hodiny (50 MHz)
--               top_rst                 -> reset (aktivni v nule)
--               top_uart_rx             -> vstup na UART (RS232)
--               top_switch_seg          -> Vstup na vyber jakej kanal chceem na sedmisegmentovkach videt
--               top_switch_vystup       -> Vstup k ovladani vystupnich dat pokud chceme vydet spatne prichozi zpravy a nebo celkove prichozi zpravy

-- Vystup   :    top_uart_tx             -> Vystupni data na UART (RS232
--               top_dir                 -> Ovladani Driveru na na sbernici
--               top_segmentovka0        -> vystup na sedmisegmentovky
--               top_segmentovka1        -> vystup na sedmisegmentovky
--               top_segmentovka2        -> vystup na sedmisegmentovky
--               top_segmentovka3        -> vystup na sedmisegmentovky
--               top_segmentovka4        -> vystup na sedmisegmentovky
--               top_segmentovka5        -> vystup na sedmisegmentovky
--               top_segmentovka6        -> vystup na sedmisegmentovky

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity TOP is
port (    top_clk               :    in   std_logic;
          top_rst               :    in   std_logic;
          top_uart_rx           :    in   std_logic;
          top_switch_seg        :    in  std_logic_vector (3 downto 0);
          top_switch_vystup     :    in  std_logic_vector (1 downto 0);

          top_uart_tx           :    out  std_logic := '1';
 --        top_dir               :    out  std_logic;
          top_segmentovka0      :    out  std_logic_vector (6 downto 0);
          top_segmentovka1      :    out  std_logic_vector (6 downto 0);
          top_segmentovka2      :    out  std_logic_vector (6 downto 0);
          top_segmentovka3      :    out  std_logic_vector (6 downto 0);
          top_segmentovka4      :    out  std_logic_vector (6 downto 0);
          top_segmentovka5      :    out  std_logic_vector (6 downto 0);
          top_segmentovka6      :    out  std_logic_vector (6 downto 0);
		  --top_magic				:    in std_logic;
		  --top_magic_led			:    out std_logic
          send_data_uart : in std_logic_vector (7 downto 0);
          send : in std_logic
);

end entity;

architecture behav of TOP is

component uart_tx
port (      --uart tx
        clk                     :   in  std_logic;
        rst                     :   in  std_logic;
        ut_tx_send              :   in  std_logic; 
        ut_tx_data              :   in  std_logic_vector(7 downto 0);       
        ut_tx                   :   out std_logic;
        ut_tx_ready             :   out std_logic 
);
end component;


component uart_rx
port  (     --uart rx
            clk             :   in std_logic;
            rst             :   in std_logic;
            ur_rx           :   in std_logic;

            ur_rx_valid     :   out std_logic := '0';
            ur_rx_data      :   out std_logic_vector(7 downto 0);
            ur_restart_sync :   buffer std_logic
);
end component;


component convert_byteTo4Byte 
port  (     --convert 
            clk                     : in    std_logic;
            rst                     : in    std_logic;
            conv_data_in            : in    std_logic_vector(7 downto 0);
            conv_valid_data_in      : in    std_logic;

            conv_data_out           : out   std_logic_vector(31 downto 0);
            conv_valid_data_out     : out   std_logic;
            conv_restart_sync       : in    std_logic
);
end component;

component CRC
port  (     --CRC
            clk                 : in std_logic;
            rst                 : in std_logic;
            CRC_data_in         : in std_logic_vector(31 downto 0);
            CRC_valid_data_in   : in std_logic;

            CRC_valid_data_out  : buffer std_logic;  -- celkovej pocet prijatych zprav
            CRC_err             : out std_logic; -- celkove spatne spocitanych CRC
            CRC_out             : out std_logic_vector(7 downto 0);
            CRC_data_out        : out std_logic_vector(23 downto 0)
);
end component;

component vypocet_chyb
port (     -- vypocet chyb 
            clk                     :   in std_logic;
            rst                     :   in std_logic;
            vypocet_data_in         :   in std_logic_vector(23 downto 0);
            vypocet_valid_data_in   :   in std_logic;
            vypocet_crc_err_in      :   in std_logic;
            vypocet_switch_vystup   :   in std_logic_vector(1 downto 0);

            vypocet_segment         :   out std_logic_vector(23 downto 0);
            vypocet_restart_sync    :   in std_logic

);
end component;

component MUX_seg

port (      --multiplexer
            clk                         :   in std_logic;
            rst                         :   in std_logic;
            mux_chyby_segment_ch1       :   in std_logic_vector(23 downto 0);
            mux_chyby_segment_ch2       :   in std_logic_vector(23 downto 0);
            mux_chyby_segment_ch3       :   in std_logic_vector(23 downto 0);
            mux_chyby_segment_ch4       :   in std_logic_vector(23 downto 0);
            mux_chyby_segment_ch5       :   in std_logic_vector(23 downto 0);
            mux_chyby_segment_ch6       :   in std_logic_vector(23 downto 0);
            mux_chyby_segment_ch7       :   in std_logic_vector(23 downto 0);
            mux_chyby_segment_ch8       :   in std_logic_vector(23 downto 0);
            mux_chyby_segment_ch9       :   in std_logic_vector(23 downto 0);
            mux_chyby_segment_ch10      :   in std_logic_vector(23 downto 0);
            mux_chyby_segment_ch11      :   in std_logic_vector(23 downto 0);
            mux_chyby_segment_ch12      :   in std_logic_vector(23 downto 0);
            mux_chyby_segment_ch13      :   in std_logic_vector(23 downto 0);
            mux_chyby_segment_ch14      :   in std_logic_vector(23 downto 0);
            mux_chyby_segment_ch15      :   in std_logic_vector(23 downto 0);
            mux_chyby_segment_ch16      :   in std_logic_vector(23 downto 0);
            mux_switch                  :   in std_logic_vector(3 downto 0);

            mux_binary_out              :   out std_logic_vector(23 downto 0)
            

);
end component;

component double_dabble
port (      --double dabble
            clk               : in std_logic;
            dd_i_Start        : in std_logic := '1';
            dd_i_Binary       : in std_logic_vector(23 downto 0);
            dd_o_DV           : out std_logic;
            dd_seg0_BNC_out   : out std_logic_vector(3 downto 0);
            dd_seg1_BNC_out   : out std_logic_vector(3 downto 0);
            dd_seg2_BNC_out   : out std_logic_vector(3 downto 0);
            dd_seg3_BNC_out   : out std_logic_vector(3 downto 0);
            dd_seg4_BNC_out   : out std_logic_vector(3 downto 0);
            dd_seg5_BNC_out   : out std_logic_vector(3 downto 0);
            dd_seg6_BNC_out   : out std_logic_vector(3 downto 0)

);
end component;

component segment is
    port (  --sedmisegmentovky
            clk                     :   in std_logic;
            rst                     :   in std_logic;
            segment_seg0_BNC_in     :   in std_logic_vector(3 downto 0);
            segment_seg1_BNC_in     :   in std_logic_vector(3 downto 0);
            segment_seg2_BNC_in     :   in std_logic_vector(3 downto 0);
            segment_seg3_BNC_in     :   in std_logic_vector(3 downto 0);
            segment_seg4_BNC_in     :   in std_logic_vector(3 downto 0);
            segment_seg5_BNC_in     :   in std_logic_vector(3 downto 0);
            segment_seg6_BNC_in     :   in std_logic_vector(3 downto 0);

            segment_seg0_n          :   out std_logic_vector(6 downto 0):="1111111";
            segment_seg1_n          :   out std_logic_vector(6 downto 0):="1111111";
            segment_seg2_n          :   out std_logic_vector(6 downto 0):="1111111";
            segment_seg3_n          :   out std_logic_vector(6 downto 0):="1111111";
            segment_seg4_n          :   out std_logic_vector(6 downto 0):="1111111";
            segment_seg5_n          :   out std_logic_vector(6 downto 0):="1111111";
            segment_seg6_n          :   out std_logic_vector(6 downto 0):="1111111"
);
end component;

--component PLL100 IS
--  PORT
--  (
--      inclk0		: IN STD_LOGIC := '0';
--      c0		    : OUT STD_LOGIC 
--  );
--END component;

signal sig_clk                     :    std_logic;
signal sig_rst                     :    std_logic;

--uart tx
signal sig_ut_tx                   :    std_logic;
signal sig_ut_tx_send              :    std_logic := '1'; 
signal sig_ut_tx_ready             :    std_logic; 
signal sig_ut_tx_data              :    std_logic_vector(7 downto 0);

--uart rx
signal sig_ur_rx                   :    std_logic;
signal sig_ur_rx_valid             :    std_logic;
signal sig_ur_rx_data              :    std_logic_vector(7 downto 0);

--convert 
signal sig_conv_data_in            :    std_logic_vector(7 downto 0);
signal sig_conv_valid_data_in      :    std_logic;
signal sig_conv_data_out           :    std_logic_vector(31 downto 0);
signal sig_conv_valid_data_out     :    std_logic;

--CRC
signal sig_CRC_data_in             :    std_logic_vector(31 downto 0);
signal sig_CRC_valid_data_in       :    std_logic;
signal sig_CRC_valid_data_out      :    std_logic ;  
signal sig_CRC_err                 :    std_logic; 
signal sig_CRC_out                 :    std_logic_vector(7 downto 0);
signal sig_CRC_data_out            :    std_logic_vector(23 downto 0);

-- vypocet chyb
signal sig_vypocet_segment         :    std_logic_vector(23 downto 0);
signal sig_vypocet_data_in         :    std_logic_vector(23 downto 0);
signal sig_vypocet_valid_data_in   :    std_logic;
signal sig_vypocet_crc_err_in      :    std_logic;
signal sig_vypocet_switch_vystup       :    std_logic_vector(1 downto 0);
--multiplexer
signal sig_chyby_segment_ch1   :    std_logic_vector(23 downto 0);
signal sig_chyby_segment_ch2   :    std_logic_vector(23 downto 0);
signal sig_chyby_segment_ch3   :    std_logic_vector(23 downto 0);
signal sig_chyby_segment_ch4   :    std_logic_vector(23 downto 0);
signal sig_chyby_segment_ch5   :    std_logic_vector(23 downto 0);
signal sig_chyby_segment_ch6   :    std_logic_vector(23 downto 0);
signal sig_chyby_segment_ch7   :    std_logic_vector(23 downto 0);
signal sig_chyby_segment_ch8   :    std_logic_vector(23 downto 0);
signal sig_chyby_segment_ch9   :    std_logic_vector(23 downto 0);
signal sig_chyby_segment_ch10  :    std_logic_vector(23 downto 0);
signal sig_chyby_segment_ch11  :    std_logic_vector(23 downto 0);
signal sig_chyby_segment_ch12  :    std_logic_vector(23 downto 0);
signal sig_chyby_segment_ch13  :    std_logic_vector(23 downto 0);
signal sig_chyby_segment_ch14  :    std_logic_vector(23 downto 0);
signal sig_chyby_segment_ch15  :    std_logic_vector(23 downto 0);
signal sig_chyby_segment_ch16  :    std_logic_vector(23 downto 0);
signal sig_mux_binary_out          :    std_logic_vector(23 downto 0);
signal sig_mux_switch              :    std_logic_vector(3 downto 0) ;

--double dabble
signal sig_dd_i_Start             :    std_logic := '1';
signal sig_dd_i_Binary            :    std_logic_vector(23 downto 0);
signal sig_dd_o_BCD               :    std_logic_vector(27 downto 0);
signal sig_dd_o_DV                :    std_logic;
signal sig_dd_seg0_BNC_out        :    std_logic_vector(3 downto 0);
signal sig_dd_seg1_BNC_out        :    std_logic_vector(3 downto 0);
signal sig_dd_seg2_BNC_out        :    std_logic_vector(3 downto 0);
signal sig_dd_seg3_BNC_out        :    std_logic_vector(3 downto 0);
signal sig_dd_seg4_BNC_out        :    std_logic_vector(3 downto 0);
signal sig_dd_seg5_BNC_out        :    std_logic_vector(3 downto 0);
signal sig_dd_seg6_BNC_out        :    std_logic_vector(3 downto 0);

  --sedmisegmentovky
signal sig_segment_seg0_BNC_in   :    std_logic_vector(3 downto 0);
signal sig_segment_seg1_BNC_in   :    std_logic_vector(3 downto 0);
signal sig_segment_seg2_BNC_in   :    std_logic_vector(3 downto 0);
signal sig_segment_seg3_BNC_in   :    std_logic_vector(3 downto 0);
signal sig_segment_seg4_BNC_in   :    std_logic_vector(3 downto 0);
signal sig_segment_seg5_BNC_in   :    std_logic_vector(3 downto 0);
signal sig_segment_seg6_BNC_in   :    std_logic_vector(3 downto 0);
signal sig_segment_seg0_n        :    std_logic_vector(6 downto 0);
signal sig_segment_seg1_n        :    std_logic_vector(6 downto 0);
signal sig_segment_seg2_n        :    std_logic_vector(6 downto 0);
signal sig_segment_seg3_n        :    std_logic_vector(6 downto 0);
signal sig_segment_seg4_n        :    std_logic_vector(6 downto 0);
signal sig_segment_seg5_n        :    std_logic_vector(6 downto 0);
signal sig_segment_seg6_n        :    std_logic_vector(6 downto 0);

signal reset_conv                :    std_logic;

signal sig_zkouska					:     std_logic := '0';

signal rozsvit                   :    std_logic := '0';

begin
--sig_clk <= top_clk;
--top_dir <= '1';

sig_mux_switch <= top_switch_seg;



--process(top_clk)
--begin
--if(top_magic = '1') then
--	top_magic_led <= '1';
--	else
--	top_magic_led <= '0';
--end if;
--end process;

top_uart_tx  <= sig_ut_tx;
--sig_ut_tx <= top_uart_rx ;
--top_uart_rx <= sig_ut_tx;

--pll1: PLL100
--    port map(
--
--        inclk0  => top_clk,
--        c0      => sig_clk
--);

uart_tx1    :uart_tx    
    port map(   
        clk             => top_clk,
        rst             => top_rst,
		ut_tx_send      => send,
        --ut_tx_send      => sig_ut_tx_send,
         ut_tx_data      => send_data_uart, 
        --ut_tx_data      => sig_ut_tx_data, 
        ut_tx           => sig_ut_tx, 
        ut_tx_ready     => sig_ut_tx_ready 
);

uart_rx1    :uart_rx    
    port map(   
        clk             => top_clk,
        rst             => top_rst, 
        ur_rx           => top_uart_rx,
        --ur_rx           => sig_ut_tx, 
        ur_rx_valid     => sig_ur_rx_valid, 
        ur_rx_data      => sig_ur_rx_data,
        ur_restart_sync => sig_zkouska
);

convert_byteTo4Byte1 :convert_byteTo4Byte   
    port map(   
        clk                  => top_clk, 
        rst                  => top_rst, 
        conv_data_in         => sig_ur_rx_data, 
        conv_valid_data_in   => sig_ur_rx_valid, 
        conv_data_out        => sig_conv_data_out, 
        conv_valid_data_out  => sig_conv_valid_data_out,
        conv_restart_sync    => sig_zkouska               
);

crc1    :crc   
    port map(  
        clk                 => top_clk, 
        rst                 => top_rst, 
        CRC_data_in         => sig_conv_data_out,
        CRC_valid_data_in   => sig_conv_valid_data_out, 
        CRC_valid_data_out  => sig_CRC_valid_data_out, 
        CRC_err             => sig_CRC_err, 
        CRC_out             => sig_CRC_out, 
        CRC_data_out        => sig_CRC_data_out
);

vypocet_chyb1 :vypocet_chyb
    port map(   
        clk                     => top_clk,
        rst                     => top_rst,
        vypocet_data_in         => sig_CRC_data_out,
        vypocet_valid_data_in   => sig_CRC_valid_data_out,
        vypocet_crc_err_in      => sig_CRC_err,
        vypocet_switch_vystup   => top_switch_vystup,
        vypocet_segment         => sig_vypocet_segment,
        vypocet_restart_sync    => sig_zkouska
);

mux_seg1 :MUX_seg
    port map(   
        clk                     => top_clk,
        rst                     => top_rst,
        mux_chyby_segment_ch1   => sig_vypocet_segment, 
        mux_chyby_segment_ch2   => sig_chyby_segment_ch2,
        mux_chyby_segment_ch3   => sig_chyby_segment_ch3,
        mux_chyby_segment_ch4   => sig_chyby_segment_ch4,
        mux_chyby_segment_ch5   => sig_chyby_segment_ch5,
        mux_chyby_segment_ch6   => sig_chyby_segment_ch6,
        mux_chyby_segment_ch7   => sig_chyby_segment_ch7,
        mux_chyby_segment_ch8   => sig_chyby_segment_ch8,
        mux_chyby_segment_ch9   => sig_chyby_segment_ch9,
        mux_chyby_segment_ch10  => sig_chyby_segment_ch10,
        mux_chyby_segment_ch11  => sig_chyby_segment_ch11,
        mux_chyby_segment_ch12  => sig_chyby_segment_ch12,
        mux_chyby_segment_ch13  => sig_chyby_segment_ch13,
        mux_chyby_segment_ch14  => sig_chyby_segment_ch14,
        mux_chyby_segment_ch15  => sig_chyby_segment_ch15,
        mux_chyby_segment_ch16  => sig_chyby_segment_ch16,
        mux_switch              => sig_mux_switch,
        mux_binary_out          => sig_mux_binary_out
);

double_dabble1 :double_dabble
    port map(
        clk                 => top_clk,
        dd_i_Start          => sig_dd_i_Start,
        dd_i_Binary         => sig_mux_binary_out,
        dd_o_DV             => sig_dd_o_DV,
        dd_seg0_BNC_out     => sig_dd_seg0_BNC_out,
        dd_seg1_BNC_out     => sig_dd_seg1_BNC_out,
        dd_seg2_BNC_out     => sig_dd_seg2_BNC_out,
        dd_seg3_BNC_out     => sig_dd_seg3_BNC_out,
        dd_seg4_BNC_out     => sig_dd_seg4_BNC_out,
        dd_seg5_BNC_out     => sig_dd_seg5_BNC_out,
        dd_seg6_BNC_out     => sig_dd_seg6_BNC_out
    );

segment1: segment
    port map(
        clk                  => top_clk,
        rst                  => top_rst,
        segment_seg0_BNC_in  => sig_dd_seg0_BNC_out, 
        segment_seg1_BNC_in  => sig_dd_seg1_BNC_out,
        segment_seg2_BNC_in  => sig_dd_seg2_BNC_out,
        segment_seg3_BNC_in  => sig_dd_seg3_BNC_out,
        segment_seg4_BNC_in  => sig_dd_seg4_BNC_out,
        segment_seg5_BNC_in  => sig_dd_seg5_BNC_out,
        segment_seg6_BNC_in  => sig_dd_seg6_BNC_out,
        segment_seg0_n       => top_segmentovka0,
        segment_seg1_n       => top_segmentovka1,
        segment_seg2_n       => top_segmentovka2,
        segment_seg3_n       => top_segmentovka3,
        segment_seg4_n       => top_segmentovka4,
        segment_seg5_n       => top_segmentovka5,
        segment_seg6_n       => top_segmentovka6
);



end;