-- author: David Bartunek
-- Modul : Vysilaci modul UART RX 
-- Vstup    :    clk                    -> hodiny (50 MHz)
--               rst                    -> reset (aktivni v nule)
--               ut_tx_send             -> rizeni vysilani
--               ut_tx_data             -> paraleln idata na vysilani
--
-- Vystup   :    ut_tx                  -> Vysilana data
--               ut_tx_ready            -> Vysilac ready

library ieee;
use IEEE.std_logic_1164.all;
use ieee.numeric_std.all;

entity uart_tx is 
generic(
    fCLK : integer := 100_000_000;
    fBAUD : integer := 10_000_000
);

port(
    --uart tx
    clk                     :   in  std_logic;                      --hodiny
    rst                     :   in  std_logic;                      --reset
    ut_tx_send              :   in  std_logic := '1';               --vysli data
    ut_tx_data              :   in  std_logic_vector(7 downto 0);   --data na vyslani

    ut_tx                   :   out std_logic := '1' ;              --vysilana data
    ut_tx_ready             :   out std_logic                       --vysilac ready
      
);
end entity;

architecture main of uart_tx is
    constant baudrate: integer := (fCLK / fBAUD);
    signal baudclk: std_logic;
    
    type state is (idle,data,stop);
    signal fsm: state := idle;

    signal data_temp: std_logic_vector(7 downto 0);
    signal datacount: unsigned(2 downto 0);
    signal txen: std_logic := '0';

    begin
        clock: process(clk)
        variable counter: integer range 0 to baudrate-1 := 0;
        begin
            if(rising_edge (clk)) then
                if(counter = baudrate-1) then
                    baudclk <= '1';
                    counter := 0;
                else
                    baudclk <= '0';
                    counter := counter + 1;
                end if;
                if(rst = '0') then
                    baudclk <= '0';
                    counter := 0;
                end if;
            end if;
        end process;

        transmit: process(clk)
        begin
            if(rising_edge (clk)) then
                if(ut_tx_send = '1' and fsm = idle) then
                    txen <= '1';
                    data_temp <= ut_tx_data;
                end if;

                if(baudclk = '1') then
                    ut_tx <= '1';
                    case  fsm is
                        when idle =>
                        ut_tx_ready <= '1';
                            if(txen = '1') then
                                datacount <= (others => '1');
                                ut_tx <= '0'; -- start bit
                                fsm <= data;
                                ut_tx_ready <= '0';
                                txen <= '0';
                            end if;
                        when data =>
                            ut_tx <= data_temp(0);
                            ut_tx_ready <= '0';
                            if(datacount = 0) then
                                fsm <= stop;
                                datacount <= (others => '1');
                            else
                                datacount <= datacount - 1;
                                data_temp <= '0' & data_temp(7 downto 1);
                            end if;
                        when stop =>
                            ut_tx <= '1'; --stop bit
                            txen <= '0';
                            fsm <= idle;
                            ut_tx_ready <= '0';
                        when others => null;
                    end case;

                    if(rst = '0') then
                        fsm <= idle;
                        ut_tx <= '1';
                        txen <= '0';
                    end if;
                end if;
            end if;
        end process;
end architecture;