-- author: David Bartunek
-- Modul : Vypocet chyb
-- Vstup    :    clk                    -> hodiny (50 MHz)
--               rst                    -> reset (aktivni v nule)
--               vypocet_data_in        -> Vstupni data
--               vypocet_valid_data_in  -> Validni vstupni data        
--               vypocet_crc_err_in     -> Vstupni spatne zpravy
--               vypocet_switch_vystup  -> Vstup na meneni typu vystupu
--
-- Vystup   :    vypocet_segment        -> Data na sedmisegmentovku binarni


library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity vypocet_chyb is
    port(
        -- vypocet chyb
        clk                     :   in std_logic;
        rst                     :   in std_logic;
        vypocet_data_in         :   in std_logic_vector(23 downto 0);
        vypocet_valid_data_in   :   in std_logic;
        vypocet_crc_err_in      :   in std_logic;
        vypocet_switch_vystup   :   in std_logic_vector(1 downto 0);
        vypocet_segment         :   out std_logic_vector(23 downto 0);
        vypocet_restart_sync    :   in std_logic
    );
end entity;

architecture behav of vypocet_chyb is

    signal err : integer := 0;
    signal cnt_data : integer := 0;
    signal prep_data : integer := 0;
    signal restart_modul : std_logic := '0';
   -- signal restart_sync : std_logic := '0';
    begin


        process(rst, vypocet_valid_data_in)
            begin
                if(rst = '0') or vypocet_restart_sync ='1' then 
                cnt_data <= 0;
                    
                else
                    if(rising_edge( vypocet_valid_data_in)) then
                        cnt_data <= cnt_data + 1;
                        if(cnt_data = 9_999_999 or restart_modul = '1') then
                            cnt_data <= 0;
                        end if;
                    end if;
                end if;
        end process;

        -- citac prichozich chyb crc_err
        process( vypocet_crc_err_in,rst)
        begin
            if(rst = '0') or vypocet_restart_sync ='1' then
                err <= 0;
            else
                if(rising_edge( vypocet_crc_err_in)) then
                    err <= err + 1;
                    if(err = 9_999_999 or restart_modul = '1') then
                        err <= 0;
                    end if;
                end if;
            end if; 
        end process;

        -- poslani hodnot crc_err na modul seg
        process(clk,rst)
            begin
                if(rst = '0') or vypocet_restart_sync ='1' then 
                    vypocet_segment <= (others => '0');
                else
                    if(rising_edge( clk)) then
                            if(vypocet_switch_vystup = "00") then
                                vypocet_segment <= std_logic_vector(to_unsigned(err, vypocet_segment'length));
                            elsif(vypocet_switch_vystup = "01") then
                                vypocet_segment <= std_logic_vector(to_unsigned(cnt_data, vypocet_segment'length));
                            end if;

                            
                    end if;
                end if;
        end process;
end architecture;