-- author: David Bartunek
-- Modul : Vypocet CRC a zhodnoceni zda prisli spatne data nebo dobre
-- Vstup    :   clk   	            -> hodiny (100Mhz)
--              rst                 -> reset (aktivni v nule)
--              CRC_data_in         -> vstupni data 32 bitu
--              CRC_valid_data_in   -> log.1 pokud jsou validni data na vstupu
--
-- Vystup   :   CRC_valid_data_out  -> log.1 pokud jsou vylidni data na vystupu    
--              CRC_err             -> log.1 pokud prislo spatne CRC      
--              CRC_out             -> prichozi CRC
--              CRC_data_out        -> vystupni data 
--
-- Info : 

-- Koeficienty CRC polynomu     x^8 + x^5 + x^4 + 1 0x8C (hex)
-- CRC sirka:                   8 bits
-- CRC shift direction:         right (little endian)
-- Vstupni data:                24 bits


library ieee;
use ieee.std_logic_1164.all;

entity CRC is 
    port(
        --CRC
        clk                 : in std_logic;
        rst                 : in std_logic :='1';
        CRC_data_in         : in std_logic_vector(31 downto 0);
        CRC_valid_data_in   : in std_logic;
        CRC_valid_data_out  : buffer std_logic := '0';  -- celkovej pocet prijatych zprav
        CRC_err             : buffer std_logic := '0'; -- celkove spatne spocitanych CRC
        CRC_out             : out std_logic_vector(7 downto 0);
        CRC_data_out        : out std_logic_vector(23 downto 0)
    );
end CRC;

architecture main of CRC is

    signal crcIN  : std_logic_vector(7 downto 0);
    signal crc_vypoctene  : std_logic_vector(7 downto 0);
    signal data : std_logic_vector(23 downto 0);
    signal data_prirazena : boolean := false;
    signal flag :std_logic := '0';
	 signal flag2 :std_logic := '0';
	 signal sig_CRC_err :std_logic := '0';
	 signal sig_CRC_valid_data_out :std_logic := '0';
     signal crc_vstupni :std_logic_vector(7 downto 0);
     signal zprava_bez_chyby : std_logic_vector(23 downto 0);
begin
    -- at na data a CRC
    -- z dat dale pocitat CRC a porovnat se vstupnim CRC
	 
	 
	CRC_valid_data_out <= sig_CRC_valid_data_out and flag and flag2;
	CRC_err <= sig_CRC_err and flag and flag2;

	 
	   process(CRC_valid_data_in)
		begin
			if (CRC_valid_data_in = '0') then
				flag2 <= '0';
			else
				flag2 <= '1';
			end if;
		end process;
		
    crc_encode: process(CRC_valid_data_in)
        begin
		  				
            if (rising_edge(CRC_valid_data_in)) then

                data(7 downto 0) <=  CRC_data_in(31 downto 24);
                data(15 downto 8) <= CRC_data_in(23 downto 16);
                data(23 downto 16) <= CRC_data_in(15 downto 8);
                crcIN <= x"00";
                crc_vstupni <= CRC_data_in(7 downto 0);

					 
                crc_vypoctene(0) <= (crcIN(0) xor crcIN(1) xor crcIN(3) xor crcIN(6) xor data(0) xor data(1) xor data(3) xor data(6) xor data(9) xor data(10) xor data(13) xor data(14) xor data(15) xor data(18) xor data(20) xor data(21));
                crc_vypoctene(1) <= (crcIN(0) xor crcIN(1) xor crcIN(2) xor crcIN(4) xor crcIN(7) xor data(0) xor data(1) xor data(2) xor data(4) xor data(7) xor data(10) xor data(11) xor data(14) xor data(15) xor data(16) xor data(19) xor data(21) xor data(22));
                crc_vypoctene(2) <= (crcIN(1) xor crcIN(2) xor crcIN(3) xor crcIN(5) xor data(1) xor data(2) xor data(3) xor data(5) xor data(8) xor data(11) xor data(12) xor data(15) xor data(16) xor data(17) xor data(20) xor data(22) xor data(23));
                crc_vypoctene(3) <= (crcIN(0) xor crcIN(1) xor crcIN(2) xor crcIN(4) xor data(0) xor data(1) xor data(2) xor data(4) xor data(10) xor data(12) xor data(14) xor data(15) xor data(16) xor data(17) xor data(20) xor data(23));
                crc_vypoctene(4) <= (crcIN(2) xor crcIN(5) xor crcIN(6) xor data(2) xor data(5) xor data(6) xor data(9) xor data(10) xor data(11) xor data(14) xor data(16) xor data(17) xor data(20));
                crc_vypoctene(5) <= (crcIN(0) xor crcIN(3) xor crcIN(6) xor crcIN(7) xor data(0) xor data(3) xor data(6) xor data(7) xor data(10) xor data(11) xor data(12) xor data(15) xor data(17) xor data(18) xor data(21));
                crc_vypoctene(6) <= (crcIN(1) xor crcIN(4) xor crcIN(7) xor data(1) xor data(4) xor data(7) xor data(8) xor data(11) xor data(12) xor data(13) xor data(16) xor data(18) xor data(19) xor data(22));
                crc_vypoctene(7) <= (crcIN(0) xor crcIN(2) xor crcIN(5) xor data(0) xor data(2) xor data(5) xor data(8) xor data(9) xor data(12) xor data(13) xor data(14) xor data(17) xor data(19) xor data(20) xor data(23));
                
				data_prirazena <= true;
					 
				if(crc_vypoctene =  crc_vstupni) then
                    sig_CRC_err <= '0';
					sig_CRC_valid_data_out <= '1';
                    CRC_out <= crc_vypoctene;
                    CRC_data_out <= CRC_data_in(31 downto 8);
				    flag <= '1';
                    zprava_bez_chyby <= CRC_data_in(31 downto 8);

                    -- prisli spatne data
                elsif (crc_vypoctene /= crc_vstupni) then
                    sig_CRC_err <= '1';
					sig_CRC_valid_data_out <= '1';
                    CRC_data_out <= zprava_bez_chyby;
                    CRC_out <= crc_vypoctene;
                    flag <= '1';
					
                end if;
            end if;
		end process;
end main ; 