-- author: David Bartunek
-- Modul : Vsrcholovej modul na Diplomovou praci 
-- Vstup    :    top_clk                 -> hodiny (50 MHz)
--               top_rst                 -> reset (aktivni v nule)
--               top_uart_rx             -> vstup na UART (RS232)
--
-- Vystup   :    top_uart_tx             -> Vystupni data na UART (RS232)
--               top_dir                 -> Ovladani Driveru na na sbernici
--               top_data                -> Výstupní registr

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity TOP_dismond is
port (  top_clk_50MHz         :    in   std_logic;
        top_rst               :    in   std_logic;
        top_uart_rx           :    in   std_logic;

        --top_dir               :     out std_logic := '1';
        --top_uart_tx           :     out  std_logic := '1';
        top_data              :     out  std_logic_vector(15 downto 0)
);

end entity;

architecture behav of TOP_dismond is

--component uart_tx
--port (      --uart tx
--        clk                     :   in  std_logic;
--        rst                     :   in  std_logic;
--        ut_tx_send              :   in  std_logic; 
--        ut_tx_data              :   in  std_logic_vector(7 downto 0); 
        
--        ut_tx                   :   out std_logic;
--        ut_tx_ready             :   out std_logic 
--);
--end component;

component uart_rx
port  (     --uart rx
            clk             :   in std_logic;
            rst             :   in std_logic;
            ur_rx           :   in std_logic;

            ur_rx_valid     :   out std_logic := '0';
            ur_rx_data      :   out std_logic_vector(7 downto 0);
            ur_restart_sync :   buffer std_logic
);
end component;

component convert_byteTo4Byte 
port  (     --convert 
            clk                     : in    std_logic;
            rst                     : in    std_logic;
            conv_data_in            : in    std_logic_vector(7 downto 0);
            conv_valid_data_in      : in    std_logic;

            conv_data_out           : out   std_logic_vector(31 downto 0);
            conv_valid_data_out     : out   std_logic;
            conv_restart_sync       : in    std_logic
);
end component;

component CRC
port  (     --CRC
            clk                 : in std_logic;
            rst                 : in std_logic;
            CRC_data_in         : in std_logic_vector(31 downto 0);
            CRC_valid_data_in   : in std_logic;

            CRC_valid_data_out  : buffer std_logic;  -- celkovy pocet prijatych zprav
            CRC_err             : out std_logic;     -- celkove spatne spocitanych CRC
            CRC_out             : out std_logic_vector(7 downto 0);
            CRC_data_out        : out std_logic_vector(23 downto 0)
);
end component;

component PLL100 IS
  PORT
  (
      inclk0		: IN STD_LOGIC := '0';
      c0		    : OUT STD_LOGIC 
  );
END component;

signal sig_clk                     :    std_logic;
signal sig_rst                     :    std_logic;

--uart tx
signal sig_ut_tx                   :    std_logic;
signal sig_ut_tx_send              :    std_logic := '1'; 
signal sig_ut_tx_ready             :    std_logic; 
signal sig_ut_tx_data              :    std_logic_vector(7 downto 0);

--uart rx
signal sig_ur_rx                   :    std_logic;
signal sig_ur_rx_valid             :    std_logic;
signal sig_ur_rx_data              :    std_logic_vector(7 downto 0);

--convert 
signal sig_conv_data_in            :    std_logic_vector(7 downto 0);
signal sig_conv_valid_data_in      :    std_logic;
signal sig_conv_data_out           :    std_logic_vector(31 downto 0);
signal sig_conv_valid_data_out     :    std_logic;

--CRC
signal sig_CRC_data_in             :    std_logic_vector(31 downto 0);
signal sig_CRC_valid_data_in       :    std_logic;
signal sig_CRC_valid_data_out      :    std_logic ;  
signal sig_CRC_err                 :    std_logic; 
signal sig_CRC_out                 :    std_logic_vector(7 downto 0);
signal sig_CRC_data_out            :    std_logic_vector(23 downto 0);

signal sig_zkouska						: std_logic;

begin
--top_dir <= '1';
--top_uart_tx  <= sig_ut_tx;
top_data <= sig_CRC_data_out(15 downto 0);


pll1: PLL100
    port map(

        inclk0  => top_clk_50MHz,
        c0      => sig_clk
);

--uart_tx1    :uart_tx    
--    port map(   
--        clk             => sig_clk,
--        rst             => top_rst,
--        ut_tx_send      => sig_ut_tx_send,
--        ut_tx_data      => sig_ut_tx_data, 
--        ut_tx           => sig_ut_tx, 
--        ut_tx_ready     => sig_ut_tx_ready 
--);

uart_rx1    :uart_rx    
    port map(   
        clk             => sig_clk,
        rst             => top_rst, 
        ur_rx           => top_uart_rx,
        ur_rx_valid     => sig_ur_rx_valid, 
        ur_rx_data      => sig_ur_rx_data,
        ur_restart_sync => sig_zkouska
);

convert_byteTo4Byte1 :convert_byteTo4Byte   
    port map(   
        clk                  => sig_clk, 
        rst                  => top_rst, 
        conv_data_in         => sig_ur_rx_data, 
        conv_valid_data_in   => sig_ur_rx_valid, 
        conv_data_out        => sig_conv_data_out, 
        conv_valid_data_out  => sig_conv_valid_data_out,
        conv_restart_sync    => sig_zkouska               
);

crc1    :crc   
    port map(  
        clk                 => sig_clk, 
        rst                 => top_rst, 
        CRC_data_in         => sig_conv_data_out,
        CRC_valid_data_in   => sig_conv_valid_data_out, 
        CRC_valid_data_out  => sig_CRC_valid_data_out, 
        CRC_err             => sig_CRC_err, 
        CRC_out             => sig_CRC_out, 
        CRC_data_out        => sig_CRC_data_out
);
end;