-- author: David Bartunek
-- Modul : Prijimaci modul UART RX 
-- Vstup    :    clk                    -> hodiny (50 MHz)
--               rst                    -> reset (aktivni v nule)
--               ur_rx                  -> vstup na UART (RS232)

-- Vystup   :    ur_rx_valid             -> Validni vystupni data
--               ur_rx_data              -> Vystupni data na paralelni

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;


entity uart_rx is 
generic(
    fCLK : integer := 100_000_000;
    fBAUD : integer := 10_000_000);

port (
    --uart rx
    clk             :   in std_logic;                                   -- hodiny
    rst             :   in std_logic;                                   -- reset
    ur_rx           :   in std_logic;                                   -- prijimana data

    ur_rx_valid     :   out std_logic := '0';
    ur_rx_data      :   out std_logic_vector(7 downto 0):= (others => '0');
    ur_restart_sync :   buffer std_logic
);
end entity;

architecture main of uart_rx is
    type state is (idle,start,data,stop);

    constant baudrate: integer := (fCLK/fBAUD);
    constant halfbaudrate: integer:= (baudrate/2);

    signal fsm : state := idle;
    signal baudclk : std_logic;
    signal data_temp : std_logic_vector(7 downto 0) := (others => '0');
    signal datacount : unsigned (2 downto 0):= (others => '1');
    signal rxflt : std_logic := '1';
    signal clken : std_logic := '0';
    signal cnt_sync : integer := 0;
    signal sync_rx  : boolean := false;
    signal cnt :integer := 0;

    begin
	 
        sync: process(clk)
        begin
            if(rising_edge(clk)) then
                if(ur_rx = '1' and sync_rx = false)then
                    cnt_sync <= cnt_sync + 1;
						  ur_restart_sync <= '0';
                else
                    cnt_sync <= 0;
                end if;

                if(cnt_sync = 5000) then
                    sync_rx <= true;
						  ur_restart_sync <= '1';
                end if;
            end if;
        end process;
		  

        filter: process(clk) is 
        
            variable flt : std_logic_vector(3 downto 0);
        begin
        if(sync_rx = true) then
            if(clk'event and clk = '1') then
                if (flt = "0000") then
                    rxflt <= '0';
                elsif (flt = "1111") then
                    rxflt <= '1';
                end if;
                flt:= flt (2 downto 0) & ur_rx;
            end if;
        end if;
        end process;

        clock: process(clk)
            variable counter : integer range 0 to baudrate-1 := 0;
        begin
        if(sync_rx = true) then
            if(clk'event and clk = '1') then
                if( counter = baudrate - 1) then
                    baudclk <= '1';
                    counter := 0;
                else
                    baudclk <= '0';
                    counter := counter + 1;
                end if;
                if (rst = '0' or ur_restart_sync = '1') then
                    baudclk <= '0';
                    counter := 0;
                end if ;
                if(clken = '0') then
                    baudclk <= '0';
                    counter := halfbaudrate;
                end if;
            end if;
        end if;
        end process;

        detect: process(clk)
            variable old_rx : std_logic := '0';
        begin
        if(sync_rx = true) then
            if(clk'event and clk = '1') then
                -- detekce sestupne hrany 
                --pokud predtim bylo 1 a ted je 0 a stav je idle
                if(old_rx = '1' and rxflt = '0' and fsm = idle) then
                    clken <= '1';
                end if;
                if(old_rx = '1' and rxflt = '1' and fsm = idle) then
                    clken <= '0'; --vypneme hodiny
                end if;
                old_rx := rxflt;

                if (rst = '0' or ur_restart_sync = '1') then
                    clken <= '0';
                    old_rx := '0';
                end if;
            end if;
        end if;
        end process;

        recieve: process(clk)
        begin
        if(sync_rx = true) then
            if(clk'event and clk = '1') then
                if(baudclk = '1') then
                    case fsm is
                        when idle =>
                            if(rxflt = '0') then
									 ur_rx_valid <= '0';
                                datacount <= (others => '1');
                                fsm <= data;
                               -- ur_rx_valid <= '0';
                            end if;

                            when data =>
                                data_temp <= rxflt & data_temp(7 downto 1);
                                if(datacount = 0) then
                                    fsm <= stop;
                                    datacount <= (others => '1');
                                else
                                    datacount <= datacount - 1;
                                end if;
                            
                            when stop => 
                                fsm <= idle;
                                
                                ur_rx_data <= data_temp;
										  --
										   ur_rx_valid <= '1';
                                 cnt <= cnt + 1;
                            when others => null;

                    end case;
                end if;
                if(rst = '0' or ur_restart_sync = '1') then
                    fsm <= idle;
                    ur_rx_valid <= '0';
                end if;

                if(cnt = 4)  then
                   -- ur_rx_valid <= '1';
                    cnt <= 0;
                else
                --    ur_rx_valid <= '0';
                end if;

            end if;
        end if;
        end process;
end architecture;


